#lang errortrace racket
(require "ast.rkt")

(define (safe-/ x y)
  (cond [(= y 0) #f]
        [else (/ x y)]))

(define (r:eval-builtin sym)
  (cond [(equal? sym '+) +]
        [(equal? sym '*) *]
        [(equal? sym '-) -]
        [(equal? sym '/) safe-/]
        [else #f]))

(define (handle-err v k)
  (define arg1 v)
    (cond
      [(false? v) v] ; In case of error, propage it
      [else (k v)]))


(define-syntax do
  (syntax-rules (<-)
    ; Only one monadic-op, return it
    [(_ mexp) mexp]
    ; A binding operation
    [(_ var <- mexp rest ...) (handle-err mexp (lambda (var) (do rest ...)))]
    ; No binding operator, just ignore the return value
    [(_ mexp rest ...)        (handle-err mexp (lambda (_) (do rest ...)))]))

(define/contract (r:eval-exp exp)
  (-> r:expression? (or/c number? #f))
  (cond
    [(r:number? exp) (r:number-value exp)]
    [(r:variable? exp) (r:eval-builtin (r:variable-name exp))]
    [(r:apply? exp)
     (do
       arg1 <- (r:eval-exp (first (r:apply-args exp)))
       arg2 <- (r:eval-exp (second (r:apply-args exp)))
       f <- (r:eval-exp (r:apply-func exp))
       (f arg1 arg2))]
    [else (error "Unknown expression:" exp)]))

; Exercise 1
; (r:eval-exp 10) ; Not OK

; (r:eval-exp (r:number 10)) ; OK





; Exercise 2
; Expression:
; (/ 1 0)
; Not OK
#|
(r:eval-exp
  (r:apply
    (r:variable '/)
    (list (r:number 1) (r:number 0))))
|#

; Exercise 3
; Expression: (+ (/ 1 0) 2)

;(+ #f 2)

(r:eval-exp
  (r:apply (r:variable '+)
    (list
      (r:apply
        (r:variable '/)
        (list (r:number 1) (r:number 0)))
      (r:number 2))))


(r:eval-exp
  (r:apply
    (r:variable 'modulo)
    (list (r:number 1) (r:number 0))))













;